// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Get and set editor content with values from external APIs.
 * This handles the different editor types Moodle supports.
 * @module    assignsubmission_vwttext/editor
 * @package   assignsubmission_vwttext
 * @copyright 2023 Bokomaru Publications
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

// Used for:
//   1. Getting the student's unsubmitted text to send to the VWT API when
//      they click on the optional "Automated evaluation" button on the
//      Add/Edit submission page.
//   2. Setting the VWT feedback when the teacher clicks the "Automated
//      evaluation" button on the grading page.
//   3. Appending or replacing the GPTZero feedback when the teacher clicks
//      the "Check for GPT" button on the grading page.
export const editor = {
    getText: function(id) {
        const element = this.getEditorElement(id);

        if (element.nodeName === 'TEXTAREA') {
            return element.value;
        } else {
            return element.innerHTML;
        }
    },
    getOnlineText: function() {
        return this.getText(this.onlineTextId);
    },
    setFeedback: function(feedback) {
        this.setText(this.feedbackId, feedback);
    },

    // Populate the assignfeedback_comments field on the teacher's grading
    // page with the response from the GPTZero API.  The response is appended
    // to the existing field contents unless there's existing GPTZero response
    // in which case this is replaced with the new response.
    setGptzeroFeedback: function(feedback) {
        const existingDivs = this.getEditorElement(this.feedbackId).getElementsByClassName('gptzero-feedback');

        if (existingDivs.length === 1) {
            this.replaceDiv(existingDivs[0], feedback);
        } else {
            // Using <div id='gptzero-feedback'>...</div> would be better but
            // Atto strips the ID attribute on save.
            this.setText(this.feedbackId,
                        this.getText(this.feedbackId) +
                        "<div class='gptzero-feedback'>" + feedback + '</div>');
        }
    },

    // Populate assignment activity's description field with the test details
    // provided by the VWT API.
    setDescription: function(description) {
        this.setText(this.descriptionId, description);
    }
};

export const init = (type) => {
    if (type === 'atto') {
        editor.onlineTextId = 'id_onlinetext_editoreditable';
        editor.feedbackId = 'id_assignfeedbackcomments_editoreditable';
        editor.descriptionId = 'id_introeditoreditable';

        editor.getEditorElement = function(id) {
            return document.getElementById(id);
        };

        editor.setText = function(id, text) {
            const element = this.getEditorElement(id);
            element.innerHTML = text;

            // XXX Without this event Atto doesn't save the text we set above
            const event = new Event("keyup");
            element.dispatchEvent(event);
        };

        editor.replaceDiv = function(innerElement, feedback) {
            innerElement.innerHTML = feedback;
            const element = document.getElementById(this.feedbackId);

            // XXX Without this event Atto doesn't save the text we set above
            const event = new Event("keyup");
            element.dispatchEvent(event);
        };
    } else if ((type === 'tinymce') || (type === 'tiny')) {
        editor.onlineTextId = "id_onlinetext_editor_ifr";
        editor.feedbackId = 'id_assignfeedbackcomments_editor_ifr';
        editor.descriptionId = 'id_introeditor_ifr';

        editor.getEditorElement = function(id) {
            const iframe = document.getElementById(id);
            return iframe.contentWindow.document.getElementById('tinymce');
        };

        editor.setText = function(id, text) {
            const element = this.getEditorElement(id);
            element.innerHTML = text;
        };

        editor.replaceDiv = function(innerElement, feedback) {
            innerElement.innerHTML = feedback;
        };
    } else if (type === 'textarea') {
        // This will probably never get used but to avoid errors we have basic
        // support for this editor type.
        editor.onlineTextId = "id_onlinetext_editor";
        editor.feedbackId = 'id_assignfeedbackcomments_editor';
        editor.descriptionId = 'id_introeditor';

        editor.getEditorElement = function(id) {
            return document.getElementById(id);
        };

        editor.setText = function(id, text) {
            const element = this.getEditorElement(id);
            element.value = text;
        };

        editor.replaceDiv = function() {
            // Do nothing.  This method should never get called since the
            // textarea editor won't return any contents as elements (i.e.
            // document.getElementByClassName('gptzero-feedback') won't find
            // anything in the editor's contents.
        };
    }
};
