// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Perform asynchronous evaluation of the student's submission by sending the
 * submission to an external API.
 * @module    assignsubmission_vwttext/evaluate
 * @package   assignsubmission_vwttext
 * @copyright 2023 Bokomaru Publications
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
import {getFeedback} from './repository';
import {getGptzeroFeedback} from './repository';
import {exception as displayException} from 'core/notification';
import Templates from 'core/templates';
import {editor} from './editor';
import {get_string as getString} from 'core/str';
import {addIconToContainerRemoveOnCompletion} from 'core/loadingicon';
import Notification from 'core/notification';

/**
 * When the student clicks the (optional) "Automated evaluation" button on
 * submission page send the text to the VWT essay checker API and show the
 * feedback to the student.
 */
export const evaluate = async() => {
    let elements = document.getElementsByName('assignsubmission_vwttext_evaluate');
    let button = elements[0];
    button.disabled = true;
    elements = document.getElementsByName('assignsubmission_vwttext_testlink');
    const testLink = elements[0].value;
    elements = document.getElementsByName('assignsubmission_vwttext_evaluation');
    const feedbackDiv = elements[0];

    elements = document.getElementsByName('assignsubmission_vwttext_loading');
    const loadingDiv = elements[0];

    getString('gettingevaluation', 'assignsubmission_vwttext')
    .then(gettingEvaluation => {
        loadingDiv.innerHTML = gettingEvaluation;
        return gettingEvaluation;
    })
    .catch();

    const promise = getFeedback(testLink, editor.getOnlineText()).fail(Notification.exception);
    addIconToContainerRemoveOnCompletion(loadingDiv, promise);

    try {
        const response = await promise;
        feedbackDiv.innerHTML = response.result;
    } catch (e) {
        // Error should be displayed to user by .fail() above.
    }

    loadingDiv.innerHTML = '';
    button.disabled = false;
};

/**
 * When the teacher clicks the "Automated evaluation" button on the grade page
 * send the student's submission (read from the <div> on the grade page) to the
 * VWT essay checker API then 1) populate the assignfeedback_comments editor
 * with the feedback, and 2) populate the grade with the API's score.
 */
export const grade = async(testLink) => {
    let elements = document.getElementsByName('assignfeedback_vwttext_evaluate');
    const evaluateButton = elements[0];
    elements = document.getElementsByName('assignfeedback_vwttext_gptzero');
    const gptzeroButton = elements[0];
    evaluateButton.disabled = true;
    gptzeroButton.disabled = true;

    // Dig up the <div> containing the student's text.  We use the hidden <div>
    // as this contains the full text (the <div> shown by default may only
    // show the truncated text with the '+' icon to view all).
    elements = document.getElementsByClassName('assignsubmission assignsubmission_onlinetext');
    elements = elements[0].getElementsByTagName('div');
    elements = elements[2].getElementsByTagName('div');
    const submissionText = elements[0].innerHTML;

    elements = document.getElementsByName('assignfeedback_vwttext_loading');
    const loadingDiv = elements[0];

    getString('gettingevaluation', 'assignsubmission_vwttext')
    .then(gettingEvaluation => {
        loadingDiv.innerHTML = gettingEvaluation;
        return gettingEvaluation;
    })
    .catch();

    // Send the text to the VWT API
    const promise = getFeedback(testLink, submissionText).fail(Notification.exception);
    addIconToContainerRemoveOnCompletion(loadingDiv, promise);

    try {
        const response = await promise;

        // Populate the grade field and assignfeedback_comments field.
        elements = document.getElementsByName('grade');
        elements[0].value = response.score;
        editor.setFeedback(response.result);
    } catch (e) {
        // Error should be displayed to user by .fail() above.
    }

    loadingDiv.innerHTML = '';
    evaluateButton.disabled = false;
    gptzeroButton.disabled = false;
};

/**
 * When the teacher clicks the "Check for GPT" button on the grade page
 * send the student's submission to the GPTZero API then either append the
 * API's feedback to the contents of the assignfeedback_comments editor, or --
 * if previous GPTZero feedback is present -- replace that feedback.
 */
export const gptzero = async() => {
    let elements = document.getElementsByName('assignfeedback_vwttext_evaluate');
    const evaluateButton = elements[0];
    elements = document.getElementsByName('assignfeedback_vwttext_gptzero');
    const gptzeroButton = elements[0];
    evaluateButton.disabled = true;
    gptzeroButton.disabled = true;

    // Dig up the <div> containing the student's text.  We use the hidden <div>
    // as this contains the full text (the <div> shown by default may only
    // show the truncated text with the '+' icon to view all).
    elements = document.getElementsByClassName('assignsubmission assignsubmission_onlinetext');
    elements = elements[0].getElementsByTagName('div');
    elements = elements[2].getElementsByTagName('div');
    const submissionText = elements[0].innerHTML;

    elements = document.getElementsByName('assignfeedback_vwttext_loading');
    const loadingDiv = elements[0];

    getString('gettingevaluation', 'assignsubmission_vwttext')
    .then(gettingEvaluation => {
        loadingDiv.innerHTML = gettingEvaluation;
        return gettingEvaluation;
    })
    .catch();

    const promise = getGptzeroFeedback(submissionText).fail(Notification.exception);
    addIconToContainerRemoveOnCompletion(loadingDiv, promise);

    try {
        const response = await promise;

        Templates.renderForPromise('assignfeedback_vwttext/gptzero', response)
            .then(({html}) => {
                editor.setGptzeroFeedback(html);
            })
            .catch((error) => displayException(error));
    } catch(e) {
        // Error should be displayed to user by .fail() above.
    }

    loadingDiv.innerHTML = '';
    evaluateButton.disabled = false;
    gptzeroButton.disabled = false;
};
