<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Calls the GPTZero API
 *
 * @package    assignfeedback_vwttext
 * @copyright  2023 Bokomaru Publications
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
namespace assignsubmission_vwttext;

require_once($CFG->libdir.'/filelib.php');

use moodle_exception;
use stdClass;

/**
 * Calls to the GPTZero API.
 */
class gptzero_api extends \curl {
    private $url = '';
    private $headers = array();

    public function __construct() {
        parent::__construct();
        $this->url = get_config('assignsubmission_vwttext', 'gptzerourl');
        $apikey = get_config('assignsubmission_vwttext', 'gptzerokey');
        $this->headers[] = "x-api-key: $apikey";
        $this->lang = 'en';

        // Hopefully max_execution_time - 1 gives us enough time to handle the
        // exception generated by a timeout.
        $timeout = ini_get('max_execution_time') - 1;

        $this->setopt(array('CURLOPT_SSL_VERIFYPEER' => 1,
                    'CURLOPT_RETURNTRANSFER' => true,
                    'CURLOPT_FOLLOWLOCATION' => true,
                    'CURLOPT_ENCODING' => "",
                    'CURLOPT_MAXREDIRS' => 10,
                    'CURLOPT_TIMEOUT' => $timeout,
                    'CURLOPT_HTTP_VERSION' => CURL_HTTP_VERSION_1_1));
    }

    /**
     * POST request to GPTZero API to evaluate specified text.
     * @param string $text Text to be evaluated by GPTZero.
     * @return array Associative array containing GPTZero's feedback
     * (json_decode(..., true)).
     * @throws api_exception if POST request's HTTP status is not 200 OK,
     * if the JSON response is empty, if the JSON is missing the 'documents'
     * element, or if JSON contains an 'error' element.
     */
    public function feedback($text) {
        $data = array('document' => $text);
        $headers = $this->headers;
        $headers[] = 'content-Type: application/json';
        $headers[] = 'accept: application/json';
        $this->setopt(array('CURLOPT_HTTPHEADER' => $headers));
        $json = json_encode($data);

        $result = $this->post($this->url, $json);

        $json = json_decode($result, true);

        if (!empty($json) && isset($json['error'])) {
            $this->error = $json['error'];
        }

        if (($this->info['http_code'] != 200) || empty($json) ||
                        isset($json['error']) || !isset($json['documents'])) {
            throw new api_exception($this, $result);
        }

        return $json;
    }

    /**
     * Is the GPTZero API available for use (i.e. has an API key been
     * specified in this plugin's configuration.
     * @return bool true if this API is available, false otherwise.
     */
    public static function available() {
        return(!empty(get_config('assignsubmission_vwttext', 'gptzerokey')));
    }
}
