<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Calls the VWT API
 *
 * @package    assignfeedback_vwttext
 * @copyright  2023 Bokomaru Publications
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
namespace assignsubmission_vwttext;

require_once($CFG->libdir.'/filelib.php');

use moodle_exception;
use stdClass;

/**
 * Calls to the VWT Grammar Check API
 */
class vwt_api extends \curl {
    private $baseURL = '';
    private $apikey = '';
    private $lang = '';
    private $headers = array();

    public function __construct() {
        parent::__construct();
        $this->baseURL = get_config('assignsubmission_vwttext', 'vwtbaseurl');
        $this->apikey = get_config('assignsubmission_vwttext', 'vwtkey');
        $this->headers[] = "vwtapikey: {$this->apikey}";
        $this->lang = 'en';

        // Hopefully max_execution_time - 1 gives us enough time to handle the
        // exception generated by a timeout.
        $timeout = ini_get('max_execution_time') - 1;

        $this->setopt(array('CURLOPT_SSL_VERIFYPEER' => 1,
                    'CURLOPT_RETURNTRANSFER' => true,
                    'CURLOPT_FOLLOWLOCATION' => true,
                    'CURLOPT_ENCODING' => "",
                    'CURLOPT_MAXREDIRS' => 10,
                    'CURLOPT_TIMEOUT' => $timeout,
                    'CURLOPT_HTTP_VERSION' => CURL_HTTP_VERSION_1_1));
    }

    /**
     * Make GET request to the VWT API endpoint .../console/essay/essay-tasks.
     * @return array of arrays each containing elements 'task_name' and
     * 'task_link'.
     */
    public function fetch_essay_list(): array {
        $cache = \cache::make('assignsubmission_vwttext', 'essays');
        $essays = $cache->get('essays');

        if ($essays === false) {
            $essays = array();
            $baseurl = get_config('assignsubmission_vwttext', 'vwtbaseurl');
            $vwtkey = get_config('assignsubmission_vwttext', 'vwtkey');
            $url = "{$this->baseURL}/essay/essay-tasks";
            $essays = $this->get_essay($url);
            $cache->set('essays', $essays);
        }

        return $essays;
    }

    /**
     * Make GET request to the VWT API endpoint
     * .../console/essay/essay-tasks.
     * @param string $tasklink Task to get links for (for example:
     * "actively-engaged-at-college").
     * @return array of arrays each containing elements 'test_name',
     * 'task_name' and 'test_link').
     */
    public function fetch_test_list(string $tasklink): array {
        $cache = \cache::make('assignsubmission_vwttext', 'tasks');
        $tasks = $cache->get($tasklink);

        if ($tasks === false) {
            $url = "{$this->baseURL}/essay/test-list/$tasklink";
            $tasks = $this->get_essay($url);
            $cache->set($tasklink, $tasks);
        }

        return $tasks;
    }

    /**
     * Make GET request to the VWT API endpoint .../console/essay/test-info.
     * @param string $testlink Test to get links for (for example:
     * "t2-house writing project 8 final exam").
     * @return array containing elements 'test_name', 'task_name' and
     * 'test_question'.
     */
    public function fetch_essay_question(string $testlink): array {
        $cache = \cache::make('assignsubmission_vwttext', 'questions');
        $testinfo = $cache->get($testlink);

        if ($testinfo === false) {
            $url = "{$this->baseURL}/essay/test-info/" .
                        rawurlencode($testlink);
            $testinfo = $this->get_essay($url);
            $cache->set($testlink, $testinfo);
        }

        return $testinfo;
    }

    /**
     * GET request to the specified endpoint.
     * @param string $url Endpoint to make GET request to, for example:
     * "https://console.virtualwritingtutor.com/console/essay/essay-tasks".
     * @return array Contents of the 'result' property in the JSON returned by
     * the VWT API.  The contents of this array will vary with the endpoint
     * called.
     * @throws moodle_exception if request's HTTP status is not 200 OK, or if
     * the JSON response is empty, or is missing the 'result' element.
     */
    private function get_essay(string $url): array {
        $this->setopt(array('CURLOPT_HTTPHEADER' => $this->headers));
        $result = $this->get($url);

        $json = json_decode($result, true);
         
        if ($this->info['http_code'] != 200) {
            throw new api_exception($this, $result);
        }

        if (empty($json) || !isset($json['result'])) {
            throw new api_exception($this, $result);
        }

        return $json['result'];
    }

    /**
     * Make POST request to VWT API endpoint .../console/essay/test-feedback
     * with student's submission and the specific test the API should evaluate
     * it against.
     * @param string $testlink Test criteria to evaluate essay against, for
     * example: "t2-house writing project 8 final exam".
     * @param string $text Student's submitted essay.
     * @return array containing elements 'score' (grade between 0 and 100) and
     * 'result' (HTML feedback generated by the VWT API).
     */
    public function feedback(string $testlink, string $text): array {
        $data = array('test_link' => $testlink, 'text' => $text);
        $headers = $this->headers;
        $headers[] = 'content-Type: application/json';
        $this->setopt(array('CURLOPT_HTTPHEADER' => $headers));
        $json = json_encode($data);
        $result = $this->post($this->baseURL . '/essay/test-feedback', $json);

        $json = json_decode($result, true);

        if ($this->info['http_code'] != 200) {
            throw new api_exception($this, $result);
        }

        if (empty($json) || !isset($json['result'])) {
            throw new api_exception($this, $result);
        }

        return $json;
    }

    /**
     * Is this API available for use (i.e. has an API key been specified in
     * this plugin's configuration.
     * @return bool true if this API is available, false otherwise.
     */
    public static function available() {
        return(!empty(get_config('assignsubmission_vwttext', 'vwtkey')));
    }
}
