<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Functions for AJAX calls to the VWT essay checker API and the GPTZero API.
 *
 * @package    assignsubmission_vwttext
 * @copyright  2023 Bokomaru Publications
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once("$CFG->libdir/externallib.php");
require_once("$CFG->dirroot/user/externallib.php");

/**
 * Functions for AJAX calls to the VWT essay checker API and the GPTZero API.
 */
class assignsubmission_vwttext_external extends external_api {
    /**
     * Returns description of method parameters
     * @return external_function_parameters
     */
    public static function get_task_list_parameters() {
        return new external_function_parameters(
            array(
                'tasklink' => new external_value(PARAM_TEXT, 'task_link from previous API (Step 1) output')
            )
        );
    }

    /**
     * Fetch the tests for a given task (i.e. call VWT API endpoint
     * .../console/essay/essay-tasks/$tasklink).
     * @param string $tasklink Task to get links for (for example:
     * "actively-engaged-at-college").
     * @return array of arrays each containing elements 'test_name',
     * 'task_name' and 'test_link').
     */
    public static function get_task_list($tasklink) {
        $vwt = new \assignsubmission_vwttext\vwt_api();
        return $vwt->fetch_test_list($tasklink);
    }

    /**
     * Returns description of method result value
     * @return external_description
     */
    public static function get_task_list_returns() {
        return new external_multiple_structure(
            new external_single_structure(
                array(
                    'test_name' => new external_value(PARAM_TEXT, 'test name'),
                    'task_name' => new external_value(PARAM_TEXT, 'task name'), // Don't think this is used?
                    'test_link' => new external_value(PARAM_TEXT, 'test link')
                )
            )
        );
    }

    /**
     * Returns description of method parameters
     * @return external_function_parameters
     */
    public static function get_test_info_parameters() {
        return new external_function_parameters(
            array(
                'testlink' => new external_value(PARAM_TEXT, 'test_link from previous API (Step 2) output')
            )
        );
    }

    /**
     * Fetch the details for a given test (i.e. call VWT API endpoint
     * .../console/essay/test-info/$testlink).
     * @param string $testlink Test to get links for (for example:
     * "t2-house writing project 8 final exam").
     * @return array containing elements 'test_name', 'task_name' and
     * 'test_question'.
     */
    public static function get_test_info($testlink) {
        $vwt = new \assignsubmission_vwttext\vwt_api();
        $question = $vwt->fetch_essay_question($testlink);
        unset($question['test_link']);
        unset($question['task_type']);
        unset($question['test_timer']);
        return $question;
    }

    /**
     * Returns description of method result value
     * @return external_description
     */
    public static function get_test_info_returns() {
        return new external_single_structure(
            array(
                'test_question' => new external_value(PARAM_RAW, 'test question'),
                'test_image' => new external_value(PARAM_TEXT, 'test image'),
                'test_name' => new external_value(PARAM_TEXT, 'test name'),
            )
        );
    }

    /**
     * Returns description of method parameters
     * @return external_function_parameters
     */
    public static function get_feedback_parameters() {
        return new external_function_parameters(
            array(
                'testlink' => new external_value(PARAM_TEXT, 'test_link from previous API (Step 2) output'),
                'text' => new external_value(PARAM_RAW, 'test_link from previous API (Step 2) output')
            )
        );
    }

    /**
     * Fetch the evaluation of the student's essay against the criteria (i.e.
     * call VWT API endpoint .../console/essay/test-feedback with essay text
     * and $testlink).
     * @param string $testlink Test criteria to evaluate essay against, for
     * example: "t2-house writing project 8 final exam".
     * @param string $text Student's submitted essay.
     * @return array containing elements 'score' (grade between 0 and 100) and
     * 'result' (HTML feedback generated by the VWT API).
     */
    public static function get_feedback($testlink, $text) {
        $text = html_to_text($text, 0, false);
        $vwt = new \assignsubmission_vwttext\vwt_api();
        $feedback = $vwt->feedback($testlink, $text);

        // Ensure score is between 0 and 100.
        $feedback['score'] = \assignfeedback_vwttext\assessor::to_grade($feedback);

        unset($feedback['max_score']);
        unset($feedback['jsonrpc']);
        unset($feedback['status']);
        return $feedback;
    }

    /**
     * Returns description of method result value
     * @return external_description
     */
    public static function get_feedback_returns() {
        return new external_single_structure(
            array(
                'score' => new external_value(PARAM_FLOAT, 'Score'),
                'result' => new external_value(PARAM_RAW, 'HTML feedback')
            )
        );
    }

    /**
     * Returns description of method parameters
     * @return external_function_parameters
     */
    public static function get_gptzero_parameters() {
        return new external_function_parameters(
            array(
                'text' => new external_value(PARAM_RAW, 'student\'s essay')
            )
        );
    }

    /**
     * Send the student's submitted essay to the GPTZero API for evaluation.
     * @param string $text Student's submitted essay.
     * @return array See get_gptzero_returns() for details.
     */
    public static function get_gptzero($text) {
        $text = html_to_text($text, 0, false);
        $gptzero = new \assignsubmission_vwttext\gptzero_api();
        $feedback = $gptzero->feedback($text);

        // The docs say to use completely_generated_prob for "... deciding
        // whether or not there is a significant use of AI in generating the
        // text" (https://gptzero.me/faq#when-and-how-should-i-use-gptzero-).
        $completelygeneratedprob = $feedback['documents'][0]['completely_generated_prob'];
        $feedback['documents'][0]['completely_generated_prob'] = round($completelygeneratedprob * 100);

        foreach ($feedback['documents'][0]['sentences'] as &$sentence) {
            $sentence['generated_prob'] = round($sentence['generated_prob'] * 100);

            // We don't use these values so discard them
            unset($sentence['perplexity']);
            unset($sentence['highlight_sentence_for_ai']);
        }

        // We don't use these values so discard them
        unset($feedback['average_generated_prob']);
        unset($feedback['overall_burstiness']);
        unset($feedback['paragraphs']);

        unset($feedback['version']);

        return $feedback;
    }

    /**
     * Returns description of method result value
     * @return external_description
     */
    public static function get_gptzero_returns() {
        return new external_single_structure(
            array(
                'documents' => new external_multiple_structure(new external_single_structure(array(
                    'completely_generated_prob' => new external_value(PARAM_FLOAT, ''),
                    'sentences' => new external_multiple_structure(new external_single_structure(array(
                        'generated_prob' => new external_value(PARAM_FLOAT, ''),
                        'sentence' => new external_value(PARAM_TEXT, ''),
                    )))
                ))
            )
        ));
    }
}
