<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * @package assignsubmission_vwttext
 * @copyright 2023 Bokomaru Publications
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

class assign_submission_vwttext extends assign_submission_plugin {
    const COMPONENT = 'assignsubmission_vwttext';

    /**
     * Get the name of the this submission plugin
     * @return string
     */
    public function get_name() {
        return get_string('vwttext', self::COMPONENT);
    }

    /**
     * Get teacher's settings for this submission plugin
     *
     * @param MoodleQuickForm $mform The form to add elements to
     * @return void
     */
    public function get_settings(MoodleQuickForm $mform) {
        if (!\assignsubmission_vwttext\vwt_api::available()) {
            // Use a group to optionally hide message (MDL-66251 workaround).
            $group = [];
            $name = self::COMPONENT . "_warning_group";
            $settingsurl = new moodle_url('/admin/settings.php',
                        array('section' => 'assignsubmission_vwttext'));
            $group[] =& $mform->createElement('static', $name, '',
                        get_string('missingvwtkey', self::COMPONENT,
                                    (string) $settingsurl));
            $mform->addGroup($group, $name);
            $mform->hideIf($name, self::COMPONENT . '_enabled', 'notchecked');
            return;
        }

        $tasklink = "";
        $testlink = "";
        $studentbutton = 1;

        if ($this->assignment->has_instance()) {
            $tasklink = $this->get_config("tasklink");
            $testlink = $this->get_config("testlink");
            $studentbutton = $this->get_config("studentbutton");
            
            if ($studentbutton === false) {
                $studentbutton = 0;
            }
        }

        global $PAGE, $CFG;
        $texteditors = explode(',', $CFG->texteditors);
        $htmleditor = get_user_preferences('htmleditor', $texteditors[0]);
        $PAGE->requires->js_call_amd('assignsubmission_vwttext/editor', 'init', [$htmleditor]);
        $PAGE->requires->js_call_amd('assignsubmission_vwttext/select');
        $PAGE->requires->js_call_amd('assignsubmission_vwttext/copy_description');

        $vwt = new \assignsubmission_vwttext\vwt_api();
        $essays = array();

        foreach ($vwt->fetch_essay_list() as $essay) {
            $essays[$essay['task_link']] = $essay['task_name'];
        }

        $name = self::COMPONENT . "_tasklink";
        $mform->addElement('select', $name, get_string('evaluationselector', self::COMPONENT), $essays, array('onchange' => "javascript:require('assignsubmission_vwttext/select').taskChanged();"));

        if (empty($tasklink)) {
            $tasklink = array_key_first($essays);
        }

        $mform->setDefault($name, $tasklink);

        $mform->hideIf($name, self::COMPONENT . '_enabled', 'notchecked');

        $vwt = new \assignsubmission_vwttext\vwt_api();
        $testlinks = array();

        // The second (tests) drop-down list is populated dynamically when the
        // first drop-down list (essays/tasks) is changed.  But Moodle's
        // Forms API removes values from a list which are not in the initial
        // definition, so the second list's value may not be available when
        // the form is submitted.  Consequently we change the value of a
        // hidden field and read that to obtain the value of the second list.
        foreach ($vwt->fetch_test_list($tasklink) as $test) {
            $tests[$test['test_link']] = $test['test_name'];
        }

        $name = self::COMPONENT . "_testlink_select";
        $mform->addElement('select', $name, '', $tests, array('onchange' => "javascript:require('assignsubmission_vwttext/select').testChanged();"));
        $mform->hideIf($name, self::COMPONENT . '_enabled', 'notchecked');

        if (!empty($testlink)) {
            $mform->setDefault($name, $testlink);
        }

        $name = self::COMPONENT . "_testlink";
        $mform->addElement('hidden', $name, $testlink);
        $mform->setType($name, PARAM_TEXT);

        // If test link not set for this assignment set hidden field to the
        // first element's value, otherwise set it to the saved value.
        $mform->setDefault($name, empty($testlink) ? array_key_first($tests) : $testlink);

        $name = self::COMPONENT . "_copy";
        $mform->addElement('button', $name, get_string('copydescription', self::COMPONENT), array('onclick' => "javascript:require('assignsubmission_vwttext/copy_description').copyDescription();"));
        $mform->hideIf($name, self::COMPONENT . '_enabled', 'notchecked');

        $name = self::COMPONENT . "_student_button";
        $mform->addElement('advcheckbox', $name, get_string('studentbutton', self::COMPONENT));
        $mform->setDefault($name, $studentbutton);
        $mform->hideIf($name, self::COMPONENT . '_enabled', 'notchecked');
    }

    /**
     * Save the settings for this submission plugin
     *
     * @param stdClass $data
     * @return bool
     */
    public function save_settings(stdClass $data) {
        $this->set_config('tasklink', $data->assignsubmission_vwttext_tasklink);
        $this->set_config('testlink', $data->assignsubmission_vwttext_testlink);
        $this->set_config('studentbutton', $data->assignsubmission_vwttext_student_button);

        return true;
    }

    /**
     * Add student's form elements for settings
     *
     * @param mixed $submission can be null
     * @param MoodleQuickForm $mform
     * @param stdClass $data
     * @return true if elements were added to the form
     */
    public function get_form_elements($submission, MoodleQuickForm $mform, stdClass $data) {
        global $PAGE;
        $PAGE->requires->js_call_amd('assignsubmission_vwttext/editsubmission', 'init');

        if (!empty($this->get_config("studentbutton"))) {
            global $CFG;
            $texteditors = explode(',', $CFG->texteditors);
            $htmleditor = get_user_preferences('htmleditor', $texteditors[0]);
            $PAGE->requires->js_call_amd('assignsubmission_vwttext/editor', 'init', [$htmleditor]);
            $PAGE->requires->js_call_amd('assignsubmission_vwttext/evaluate');
            $name = self::COMPONENT . '_evaluate';
            $mform->addElement('button', $name, get_string('automatedevaluation', self::COMPONENT), array('onclick' => "javascript:require('assignsubmission_vwttext/evaluate').evaluate();"));
            $name = self::COMPONENT . '_testlink';
            $testlink = $this->get_config("testlink");
            $mform->addElement('hidden', $name, $testlink);
            $mform->setType($name, PARAM_TEXT);

            $name = self::COMPONENT . '_evaluation';
            $mform->addElement('html', '<div name="' . $name . '"></div>');
        }

        $name = self::COMPONENT . '_loading';
        $mform->addElement('html', '<div name="' . $name . '"></div>');

        return true;
    }

    /**
     * Always return true as nothing is submitted for this plugin.
     *
     * @param stdClass $data The submission data
     * @return bool
     */
    public function submission_is_empty(stdClass $data) {
        return true;
    }

    /**
     * Return the plugin configs for external functions.
     *
     * @return array the list of settings
     * @since Moodle 3.2
     */
    public function get_config_for_external() {
        return (array) $this->get_config();
    }
}

// Plugin supports Moodle 3.8 which supports PHP versions without array_key_first().
if (!function_exists('array_key_first')) {
    function array_key_first(array $arr) {
        foreach($arr as $key => $unused) {
            return $key;
        }
        return NULL;
    }
}
